/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.engine;

import com.carlosbcruz.model.Board;

/**
 * Analyze what solutions can be found a specific board.
 *
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class BoardAnalysis {

    private Board internalBoard;

    private int possibilities[][][];

    /**
     * Receives a board to work on.
     * 
     * @param newBoard
     */
    public BoardAnalysis(Board newBoard) {

        // Set the board
        internalBoard = newBoard;

        // Filll up the array structure to hold the possibilities
        possibilities = new int[9][][];
        for (int i = 0; i < 9; i++)
            possibilities[i] = new int[9][];

        setPossibilities();
    }

    /**
     * Generate and store all the possibilites of all positions of the board.
     */
    private void setPossibilities() {

        // Follow all the rows
        for (int x = 1; x <= 9; x++)

            // Follow all the columns
            for (int y = 1; y <= 9; y++) {

                possibilities[x - 1][y - 1] = generatePossibilities(x, y);
            }
    }

    /**
     * Analyze if the board is a valid one.
     */
    public boolean isValid() {

        // Follow all the rows
        for (int x = 1; x <= 9; x++) {

            // Follow all the columns
            for (int y = 1; y <= 9; y++) {

                int valueEntered = internalBoard.getValue(x, y);

                if (valueEntered != 0) {

                    // Search the column to see if the value occurs
                    for (int currentX = 1; currentX <= 9; currentX++) {
                        if (currentX != x
                            && internalBoard.getValue(currentX, y) == valueEntered) {
                            return false;
                        }
                    }

                    // Search the row to see if the value occurs
                    for (int currentY = 1; currentY <= 9; currentY++) {
                        if (currentY != y
                            && internalBoard.getValue(x, currentY) == valueEntered) {
                            return false;
                        }
                    }

                    // Quadrant 1
                    if (x >= 1 && x <= 3 && y >= 1 && y <= 3)
                        for (int r = 1; r <= 3; r++)
                            for (int c = 1; c <= 3; c++)
                                if (x != r
                                    && y != c
                                    && internalBoard.getValue(r, c) == valueEntered) {
                                    return false;
                                }

                    // Quadrant 2
                    if (x >= 1 && x <= 3 && y >= 4 && y <= 6)
                        for (int r = 1; r <= 3; r++)
                            for (int c = 4; c <= 6; c++)
                                if (x != r
                                    && y != c
                                    && internalBoard.getValue(r, c) == valueEntered) {
                                    return false;
                                }

                    // Quadrant 3
                    if (x >= 1 && x <= 3 && y >= 7 && y <= 9)
                        for (int r = 1; r <= 3; r++)
                            for (int c = 7; c <= 9; c++)
                                if (x != r
                                    && y != c
                                    && internalBoard.getValue(r, c) == valueEntered) {
                                    return false;
                                }

                    // Quadrant 4
                    if (x >= 4 && x <= 6 && y >= 1 && y <= 3)
                        for (int r = 4; r <= 6; r++)
                            for (int c = 1; c <= 3; c++)
                                if (x != r
                                    && y != c
                                    && internalBoard.getValue(r, c) == valueEntered) {
                                    return false;
                                }

                    // Quadrant 5
                    if (x >= 4 && x <= 6 && y >= 4 && y <= 6)
                        for (int r = 4; r <= 6; r++)
                            for (int c = 4; c <= 6; c++)
                                if (x != r
                                    && y != c
                                    && internalBoard.getValue(r, c) == valueEntered) {
                                    return false;
                                }
                    // Quadrant 6
                    if (x >= 4 && x <= 6 && y >= 7 && y <= 9)
                        for (int r = 4; r <= 6; r++)
                            for (int c = 7; c <= 9; c++)
                                if (x != r
                                    && y != c
                                    && internalBoard.getValue(r, c) == valueEntered) {
                                    return false;
                                }

                    // Quadrant 7
                    if (x >= 7 && x <= 9 && y >= 1 && y <= 3)
                        for (int r = 7; r <= 9; r++)
                            for (int c = 1; c <= 3; c++)
                                if (x != r
                                    && y != c
                                    && internalBoard.getValue(r, c) == valueEntered) {
                                    return false;
                                }

                    // Quadrant 8
                    if (x >= 7 && x <= 9 && y >= 4 && y <= 6)
                        for (int r = 7; r <= 9; r++)
                            for (int c = 4; c <= 6; c++)
                                if (x != r
                                    && y != c
                                    && internalBoard.getValue(r, c) == valueEntered) {
                                    return false;
                                }

                    // Quadrant 9
                    if (x >= 7 && x <= 9 && y >= 7 && y <= 9)
                        for (int r = 7; r <= 9; r++)
                            for (int c = 7; c <= 9; c++)
                                if (x != r
                                    && y != c
                                    && internalBoard.getValue(r, c) == valueEntered) {
                                    return false;
                                }
                }
            }
        }
        return true;
    }

    /**
     * Generate all the possible number that can be inserted in the informed row
     * and column.
     * 
     * @param x
     *            row
     * @param y
     *            column
     * @return the possible values in a array of int of 9 positions. The values
     *         are at the very first positions and the remaining positions are
     *         filled with zeros.
     */
    private int[] generatePossibilities(int x, int y) {

        int newPossibilities[] = new int[9];
        int newPossibilitiesArrayPosition = 0;

        if (internalBoard.getValue(x, y) == 0) {
            // For each value possible, analyze if it can be assigned to the
            // indicated row by column
            for (int value = 1; value <= 9; value++) {

                boolean valueCanBeAssigned = true;

                // Quadrant 1
                if (x >= 1 && x <= 3 && y >= 1 && y <= 3)
                    for (int r = 1; r <= 3; r++)
                        for (int c = 1; c <= 3; c++)
                            if (internalBoard.getValue(r, c) == value) {
                                valueCanBeAssigned = false;
                                break;
                            }

                // Quadrant 2
                if (x >= 1 && x <= 3 && y >= 4 && y <= 6)
                    for (int r = 1; r <= 3; r++)
                        for (int c = 4; c <= 6; c++)
                            if (internalBoard.getValue(r, c) == value) {
                                valueCanBeAssigned = false;
                                break;
                            }

                // Quadrant 3
                if (x >= 1 && x <= 3 && y >= 7 && y <= 9)
                    for (int r = 1; r <= 3; r++)
                        for (int c = 7; c <= 9; c++)
                            if (internalBoard.getValue(r, c) == value) {
                                valueCanBeAssigned = false;
                                break;
                            }

                // Quadrant 4
                if (x >= 4 && x <= 6 && y >= 1 && y <= 3)
                    for (int r = 4; r <= 6; r++)
                        for (int c = 1; c <= 3; c++)
                            if (internalBoard.getValue(r, c) == value) {
                                valueCanBeAssigned = false;
                                break;
                            }

                // Quadrant 5
                if (x >= 4 && x <= 6 && y >= 4 && y <= 6)
                    for (int r = 4; r <= 6; r++)
                        for (int c = 4; c <= 6; c++)
                            if (internalBoard.getValue(r, c) == value) {
                                valueCanBeAssigned = false;
                                break;
                            }
                // Quadrant 6
                if (x >= 4 && x <= 6 && y >= 7 && y <= 9)
                    for (int r = 4; r <= 6; r++)
                        for (int c = 7; c <= 9; c++)
                            if (internalBoard.getValue(r, c) == value) {
                                valueCanBeAssigned = false;
                                break;
                            }

                // Quadrant 7
                if (x >= 7 && x <= 9 && y >= 1 && y <= 3)
                    for (int r = 7; r <= 9; r++)
                        for (int c = 1; c <= 3; c++)
                            if (internalBoard.getValue(r, c) == value) {
                                valueCanBeAssigned = false;
                                break;
                            }

                // Quadrant 8
                if (x >= 7 && x <= 9 && y >= 4 && y <= 6)
                    for (int r = 7; r <= 9; r++)
                        for (int c = 4; c <= 6; c++)
                            if (internalBoard.getValue(r, c) == value) {
                                valueCanBeAssigned = false;
                                break;
                            }

                // Quadrant 9
                if (x >= 7 && x <= 9 && y >= 7 && y <= 9)
                    for (int r = 7; r <= 9; r++)
                        for (int c = 7; c <= 9; c++)
                            if (internalBoard.getValue(r, c) == value) {
                                valueCanBeAssigned = false;
                                break;
                            }

                // Search the column to see if the value occurs
                for (int currentX = 1; currentX <= 9; currentX++) {
                    if (internalBoard.getValue(currentX, y) == value) {
                        valueCanBeAssigned = false;
                        break;
                    }
                }

                // Search the row to see if the value occurs
                for (int currentY = 1; currentY <= 9; currentY++) {
                    if (internalBoard.getValue(x, currentY) == value) {
                        valueCanBeAssigned = false;
                        break;
                    }
                }

                // If can be assigned than store it
                if (valueCanBeAssigned) {
                    newPossibilities[newPossibilitiesArrayPosition] = value;
                    newPossibilitiesArrayPosition++;
                }
            }
        }
        return newPossibilities;
    }

    /**
     * Indicate a possibility that was applied.
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    public class CurrentPossibility {

        private int x;

        private int y;

        private int z;

        /**
         * Indicate the current applied possibility.
         * 
         * @param x
         *            row
         * @param y
         *            column
         * @param z
         *            possibility
         */
        CurrentPossibility(int x, int y, int z) {

            this.x = x;
            this.y = y;
            this.z = z;
        }

        /**
         * Get row.
         * 
         * @return Returns the x.
         */
        public int getX() {

            return x;
        }

        /**
         * get column.
         * 
         * @return Returns the y.
         */
        public int getY() {

            return y;
        }

        /**
         * get possibility.
         * 
         * @return Returns the z.
         */
        public int getZ() {

            return z;
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Object#toString()
         */
        public String toString() {

            return "(" + x + "," + y + ")" + " possibility: " + z;
        }
    }

    /**
     * Select the row and column with shorter number of possibilities and apply
     * the first value to the board.
     */
    public CurrentPossibility setValueFromShorterPossibilitie() {

        // Shorter possibility array
        int minimumX = 0, minimumY = 0, size = 10;

        // Follow all the rows
        for (int x = 1; x <= 9; x++) {

            // Follow all the columns
            for (int y = 1; y <= 9; y++) {

                int numberOfPossibilities = 0;
                // Write the possibilities of that position
                for (int z = 0; z < possibilities[x - 1][y - 1].length; z++) {

                    // Stop when find a zero
                    if (possibilities[x - 1][y - 1][z] == 0)
                        break;

                    numberOfPossibilities++;
                }

                if (numberOfPossibilities != 0 && numberOfPossibilities < size) {
                    minimumX = x;
                    minimumY = y;
                    size = numberOfPossibilities;
                }
            }
        }

        // If there is no possibilities return false
        if (size == 10)
            return null;

        /*
         * System.out.println("Minimum = (" + minimumX + "," + minimumY + ")
         * Size = " + size);
         */
        internalBoard.setValue(minimumX, minimumY,
            possibilities[minimumX - 1][minimumY - 1][0]);

        //System.out.println(internalBoard.toString());

        return new CurrentPossibility(minimumX, minimumY, 0);

    }

    /**
     * Apply the next value on the possibility array of a specific position
     * @param currentPossibility The class that indicates the previous possibility
     * @return the possibility applied.
     */
    public CurrentPossibility applyNextPossibility(
        CurrentPossibility currentPossibility) {

        // Create the next possibility
        CurrentPossibility newCurrentPossibility = new CurrentPossibility(
            currentPossibility.getX(), currentPossibility.getY(),
            currentPossibility.getZ() + 1);

        // Get its value
        int value = possibilities[newCurrentPossibility.getX() - 1][newCurrentPossibility
                .getY() - 1][newCurrentPossibility.getZ()];

        // If there is not a next value then return
        if (value == 0)
            return null;

        // Apply the next value to the board
        internalBoard.setValue(newCurrentPossibility.getX(),
            newCurrentPossibility.getY(), value);

        // Return the current applied possibility
        return newCurrentPossibility;

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {

        StringBuffer output = new StringBuffer(3000);

        // Follow all the rows
        for (int x = 1; x <= 9; x++)

            // Follow all the columns
            for (int y = 1; y <= 9; y++) {

                // Get possibilities
                int[] currentPossibility = generatePossibilities(x, y);

                // Write the position analyzed
                output.append("(" + x + "," + y + ")");

                // Write the possibilities of that position
                for (int z = 0; z < currentPossibility.length; z++) {

                    // Stop when find a zero
                    if (currentPossibility[z] == 0)
                        break;

                    output.append(" " + currentPossibility[z]);

                }

                output.append(System.getProperty("line.separator"));
            }

        return output.toString();
    }
}
