/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.engine;

import com.carlosbcruz.model.Board;

/**
 * Apply a backtrack algorith to solve the board problem.
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class BoardSolver extends Thread {

    Board boardToBeSolved;

    Board lastFoundSolution;

    boolean exitRequested = false;

    private boolean stopAtFirstSolution = false;

    private int stepsAnalyzed;

    private int numberOfSolutions;

    /**
     * Request the solver to exit.
     * 
     * @param exitRequested
     *            The exitRequested to set.
     */
    public void setExitRequested(boolean exitRequested) {

        this.exitRequested = exitRequested;
    }

    /**
     * Thread that will work on a specific board.
     * 
     * @param boardToBeSolver
     */
    public BoardSolver(Board boardToBeSolver) {

        this.boardToBeSolved = boardToBeSolver;
    }

    /**
     * Inform the number of solutions found.
     * 
     * @return Returns the numberOfSolutions.
     */
    public int getNumberOfSolutions() {

        return numberOfSolutions;
    }

    /**
     * Indicate if the flat stop after a solution is set or not.
     * 
     * @return Returns the stopAtFirstSolution.
     */
    public boolean isStopAtFirstSolution() {

        return stopAtFirstSolution;
    }

    /**
     * set if the it is required to stop after a solution.
     * 
     * @param stopAtFirstSolution
     *            The stopAtFirstSolution to set.
     */
    public void setStopAtFirstSolution(boolean stopAtFirstSolution) {

        this.stopAtFirstSolution = stopAtFirstSolution;
    }

    /**
     * Get the number of steps executed.
     * 
     * @return Returns the stepsAnalyzed.
     */
    public int getStepsAnalyzed() {

        return stepsAnalyzed;
    }

    /**
     * Return the last found soulution.
     * 
     * @return Returns the lastFoundSolution.
     */
    public Board getLastFoundSolution() {

        return lastFoundSolution;
    }

    /**
     * Start the seek for a solution.
     * 
     * @param board
     *            the problem describe in the board
     * @return true if it is solved and false otherwise. Also return false if
     *         requested to stop.
     */
    public boolean solveBoard(Board board) {

        stepsAnalyzed++;

        if (exitRequested)
            return false;

        // If solved return true;
        if (board.isSolved()) {
            numberOfSolutions++;
            lastFoundSolution = board;
            return stopAtFirstSolution;
        }

        // Clone it
        Board boardClone = board.createClone();

        // Analyze it
        BoardAnalysis boardAnalysis = new BoardAnalysis(boardClone);

        // Follow the possibilities of a specific position
        BoardAnalysis.CurrentPossibility currentPossibility = boardAnalysis
                .setValueFromShorterPossibilitie();

        while (currentPossibility != null) {

            // Try to solve this problem
            boolean solved = solveBoard(boardClone);

            if (exitRequested)
                return false;

            if (solved) {
                return true;
            }

            currentPossibility = boardAnalysis
                    .applyNextPossibility(currentPossibility);

        }

        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Runnable#run()
     */
    public void run() {

        // Start the board
        solveBoard(boardToBeSolved);
    }
}
