/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.model;

/**
 * Represent a board
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class Board implements Cloneable {

    // Simple unidirectional array but represents a 9 x 9 board
    private int boardPositions[] = new int[9 * 9];

    /**
     * Set a value in the board.
     * 
     * @param x
     *            row
     * @param y
     *            column
     * @param newValue
     *            new value to be inserted in the board at x row by y column
     */
    public void setValue(int x, int y, int newValue) {

        x--;
        y--;

        boardPositions[x * 9 + y] = newValue;

    }

    /**
     * Consult a value on a position of the board.
     * 
     * @param x
     *            row
     * @param y
     *            colunn
     * @return stored value
     */
    public int getValue(int x, int y) {

        x--;
        y--;

        return boardPositions[x * 9 + y];
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {

        StringBuffer output = new StringBuffer(300);

        // Follow the rows
        for (int x = 1; x < 10; x++) {

            // Add a line at each 3 rows
            if (((x - 1) % 3) == 0)
                output.append(System.getProperty("line.separator"));

            // Follow the columns
            for (int y = 1; y < 10; y++) {

                // Add a space at each 3 columns
                if (((y - 1) % 3) == 0)
                    output.append(" ");

                // Append the value
                output.append(getValue(x, y));
                output.append(" ");
            }

            output.append(System.getProperty("line.separator"));
        }
        return output.toString();
    }

    /**
     * See if the board problem is solved.
     * 
     * @return true or false
     */
    public boolean isSolved() {

        // Follow the rows
        for (int x = 1; x < 10; x++)
            // Follow the columns
            for (int y = 1; y < 10; y++)
                if (getValue(x, y) == 0)
                    return false;

        return true;
    }

    /**
     * Generate a copy of the current board.
     * 
     * @return a new board copy
     */
    public Board createClone() {

        Board newBoard = new Board();

        // Follow the rows
        for (int x = 1; x < 10; x++)
            // Follow the columns
            for (int y = 1; y < 10; y++)
                newBoard.setValue(x, y, getValue(x, y));

        return newBoard;
    }
}
