/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.view;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

import com.carlosbcruz.engine.BoardAnalysis;
import com.carlosbcruz.engine.BoardSolver;
import com.carlosbcruz.model.Board;

/**
 * Controls the board manipulation.
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class BoardView extends JPanel {

    private static final long serialVersionUID = 1L;

    //	Board initialBoard = new Board();

    NumericTextField textField[][] = new NumericTextField[9][];

    JPanel threadListPanel = new JPanel();

    JButton analyzeButton = new JButton("Analyze");

    JButton removeButton = new JButton("Remove");

    ArrayList currentProblems = new ArrayList();

    /**
     * Create the entire windows with all the components
     */
    public BoardView(final RepaintListener repaintComponent) {

        // Get Content page and set layout
        setLayout(new BorderLayout());

        // Finalize the creation of bidimentional array of
        // pointers to text fields to enter only numbers.
        for (int i = 0; i < 9; i++) {
            textField[i] = new NumericTextField[9];
        }

        // Panel to hold only the textfields on a grid layout.
        JPanel textFieldPanel = new JPanel();
        textFieldPanel.setLayout(new GridLayout(11, 11));

        // Create the text fields and add to the main window
        for (int r = 0; r < 9; r++) {
            for (int c = 0; c < 9; c++) {
                textField[r][c] = new NumericTextField(1);
                textField[r][c].setPreferredSize(new Dimension(20, 20));
                textFieldPanel.add(textField[r][c]);
                if (c == 2 || c == 5) {
                    textFieldPanel.add(new JLabel(" "));
                }
                    
            }
            if (r == 2 || r == 5) {
                for (int i = 0; i < 11; i++) {
                    textFieldPanel.add(new JLabel(" "));    
                }
            }
        }
        add(textFieldPanel, BorderLayout.CENTER);

        // Create a penal to hold the thread lists.
        threadListPanel.setLayout(new GridLayout(10, 1));
        add(threadListPanel, BorderLayout.EAST);

        // Create the lower panel to hold the buttons analyze and remove
        JPanel buttonsPanel = new JPanel(new BorderLayout());
        buttonsPanel.add(analyzeButton, BorderLayout.WEST);
        buttonsPanel.add(removeButton, BorderLayout.EAST);
        buttonsPanel.add(new JLabel(
            "Author: Carlos Fernando Bella Cruz (www.carlosbcruz.com)"),
            BorderLayout.SOUTH);
        add(buttonsPanel, BorderLayout.SOUTH);

        threadListPanel
                .add(new JLabel(
                    "Boards being analized:                                                                                                      "));
        // Create the event for remove button
        removeButton.addActionListener(new ActionListener() {

            /* (non-Javadoc)
             * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
             */
            public void actionPerformed(ActionEvent e) {

                // Store the checked ones in the array to be removed.
                ArrayList toBeRemoved = new ArrayList();

                // Iterate over the thread list
                Iterator iterator = currentProblems.iterator();
                while (iterator.hasNext()) {
                    // Retrieve the thread
                    RunningSolution solution = (RunningSolution)iterator.next();
                    // Should be removed?
                    if (solution.isChecked()) {
                        // First request the thread to stop
                        solution.requestTerminate();
                        // Remove it from panel and add to the removal list.
                        toBeRemoved.add(solution);
                        threadListPanel.remove(solution);
                    }
                }
                // Remove
                currentProblems.removeAll(toBeRemoved);

                repaintComponent.repaintRequested();
            }
        });

        // Create the event for analyze button
        analyzeButton.addActionListener(new ActionListener() {

            /* (non-Javadoc)
             * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
             */
            public void actionPerformed(ActionEvent e) {

                if (currentProblems.size() == 6) {
                    JOptionPane
                            .showMessageDialog(null,
                                "You have reached the maximum of 6 solutions. Please remove any solution.");
                    return;
                }
                // Transfer the information on the window to a temporary board
                Board temporaryBoard = new Board();
                for (int r = 0; r < 9; r++)
                    for (int c = 0; c < 9; c++) {
                        String text = textField[r][c].getText();
                        if (text != null && !text.equals("")) {
                            try {
                                temporaryBoard
                                        .setValue(r + 1, c + 1, textField[r][c]
                                                .getNumberValue().intValue());
                            }
                            catch(ParseException e1) {
                                e1.printStackTrace();
                            }
                        }
                    }

                // Verify if it is a valid board
                BoardAnalysis boardAnalysis = new BoardAnalysis(temporaryBoard);
                if (!boardAnalysis.isValid()) {
                    JOptionPane.showMessageDialog(null,
                        "This is not a valid initial board");

                } else {
                    // If it is a valid board then create a thread and
                    // add it to to the thread list
                    RunningSolution runningSolution = new RunningSolution(
                        temporaryBoard);
                    currentProblems.add(runningSolution);
                    threadListPanel.add(runningSolution);

                    repaintComponent.repaintRequested();
                }
            }
        });
    }

    /**
     * Represents a thread and control seek for a solution
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    class RunningSolution extends JPanel {

        private static final long serialVersionUID = 1L;

        Board internalBoard;

        BoardSolver boardSolver;

        JLabel numberOfSolutionsLabel = new JLabel("Solutions: ");

        JLabel numberOfStepsLabel = new JLabel("Steps: ");

        JLabel numberOfSolutions = new JLabel("0");

        JLabel numberOfSteps = new JLabel("0");

        JButton stopButton = new JButton("Stop");

        JButton showSolutionButton = new JButton("Show one solution");

        JCheckBox checkBox = new JCheckBox();

        boolean checked = false;

        Timer timer = new Timer(1000, new TimerActionListener());

        /**
         * Controls the events form the Tiver to update the thread on the
         * interface
         * 
         * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
         */
        class TimerActionListener implements ActionListener {

            /*
             * (non-Javadoc)
             * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
             */
            public void actionPerformed(ActionEvent e) {

                numberOfSolutions.setText(Integer.toString(boardSolver
                        .getNumberOfSolutions()));
                numberOfSteps.setText(Integer.toString(boardSolver
                        .getStepsAnalyzed()));
            }
        }

        /**
         * Create the thread panel and prepare the thread.
         * 
         * @param board
         */
        RunningSolution(Board board) {

            internalBoard = board;
            boardSolver = new BoardSolver(board);

            add(checkBox);
            add(numberOfSolutionsLabel);
            add(numberOfSolutions);
            add(numberOfStepsLabel);
            add(numberOfSteps);
            add(stopButton);
            add(showSolutionButton);

            // It is necessary to press start
            stopButton.setEnabled(false);

            // If the check box is pressend the update the
            // internal status
            checkBox.addActionListener(new ActionListener() {

                /* (non-Javadoc)
                 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
                 */
                public void actionPerformed(ActionEvent e) {

                    checked = checkBox.isSelected();
                }
            });

            showSolutionButton.addActionListener(new ActionListener() {

                /* (non-Javadoc)
                 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
                 */
                public void actionPerformed(ActionEvent e) {

                    // Retrieve the last solution
                    Board lastSolution = boardSolver.getLastFoundSolution();
                    // If exists than send to standard out
                    if (lastSolution == null)
                        JOptionPane.showMessageDialog(null,
                            "No solution found so far");
                    else {
                        JOptionPane.showMessageDialog(null, lastSolution
                                .toString());
                    }
                }
            });

            // Stop the thread
            stopButton.addActionListener(new ActionListener() {

                /* (non-Javadoc)
                 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
                 */
                public void actionPerformed(ActionEvent e) {

                    // Request the thread to stop
                    boardSolver.setExitRequested(true);
                    // Update the buttons
                    stopButton.setEnabled(false);
                    
                    // Retrieve the last solution
                    Board lastSolution = boardSolver.getLastFoundSolution();
                    // If exists than send to standard out
                    if (lastSolution != null) {
                        JOptionPane.showMessageDialog(null, lastSolution
                                .toString());
                    }
                    
                    // Attach a new board
                    boardSolver = new BoardSolver(internalBoard);
                }
            });

            timer.start();
            boardSolver.start();
            stopButton.setEnabled(true);
        }

        /**
         * Repass the temination message to the board solver.
         */
        public void requestTerminate() {

            if (boardSolver != null)
                boardSolver.setExitRequested(true);
        }

        /**
         * @return Returns the checked.
         */
        public boolean isChecked() {

            return checked;
        }
    }
}
