/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.view;

import java.awt.Toolkit;
import java.text.DecimalFormat;
import java.text.ParseException;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.Document;

/**
 * Control the numberic text input.
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class NumericTextField extends JTextField implements
    NumericPlainDocument.InsertErrorListener {

    private int maxDigits = 0;

    private static final long serialVersionUID = 1L;

    /**
     * Constructor
     * @param maxDigits
     */
    public NumericTextField(int maxDigits) {

        this(null, 0, null, maxDigits);
    }

    /**
     * Constructor
     * @param text
     * @param columns
     * @param format
     * @param maxDigits
     */
    public NumericTextField(String text, int columns, DecimalFormat format,
        int maxDigits) {

        super(null, text, columns);

        NumericPlainDocument numericDoc = (NumericPlainDocument)getDocument();
        if (format != null) {
            numericDoc.setFormat(format);
        }

        numericDoc.addInsertErrorListener(this);

        this.maxDigits = maxDigits;
    }

    /**
     * Constructor
     * @param columns
     * @param format
     * @param maxDigits
     */
    public NumericTextField(int columns, DecimalFormat format, int maxDigits) {

        this(null, columns, format, maxDigits);
    }

    /**
     * Constructor
     * @param text
     * @param maxDigits
     */
    public NumericTextField(String text, int maxDigits) {

        this(text, 0, null, maxDigits);
    }

    /**
     * Constructor
     * @param text
     * @param columns
     * @param maxDigits
     */
    public NumericTextField(String text, int columns, int maxDigits) {

        this(text, columns, null, maxDigits);
    }

    /**
     * @param format
     */
    public void setFormat(DecimalFormat format) {

        ((NumericPlainDocument)getDocument()).setFormat(format);
    }

    /**
     * @return the formatter
     */
    public DecimalFormat getFormat() {

        return ((NumericPlainDocument)getDocument()).getFormat();
    }

    /**
     * Notify change of format attributes.
     */
    public void formatChanged() {

        setFormat(getFormat());
    }

    /**
     * Methods to get the field value
     * @return get the content as a long
     * @throws ParseException
     */
    public Long getLongValue() throws ParseException {

        return ((NumericPlainDocument)getDocument()).getLongValue();
    }

    /**
     * @return get the content as a double
     * @throws ParseException
     */
    public Double getDoubleValue() throws ParseException {

        return ((NumericPlainDocument)getDocument()).getDoubleValue();
    }

    /**
     * @return get the content as a Number
     * @throws ParseException
     */
    public Number getNumberValue() throws ParseException {

        return ((NumericPlainDocument)getDocument()).getNumberValue();
    }

    /**
     * Methods to install numeric values
     * @param number
     */
    public void setValue(Number number) {

        setText(getFormat().format(number));
    }

    /**
     * @param l
     */
    public void setValue(long l) {

        setText(getFormat().format(l));;
    }

    /**
     * @param d
     */
    public void setValue(double d) {

        setText(getFormat().format(d));
    }

    /**
     * Format the value according to the format string
     * @throws ParseException
     */
    public void normalize() throws ParseException {

        setText(getFormat().format(getNumberValue()));
    }

    /* (non-Javadoc)
     * @see com.carlosbcruz.view.NumericPlainDocument.InsertErrorListener#insertFailed(com.carlosbcruz.view.NumericPlainDocument, int, java.lang.String, javax.swing.text.AttributeSet)
     */
    public void insertFailed(NumericPlainDocument doc, int offset, String str,
        AttributeSet a) {

        // Override to handle insertion error
        // By default, just beep
        Toolkit.getDefaultToolkit().beep();
    }

    /* (non-Javadoc)
     * @see javax.swing.JTextField#createDefaultModel()
     */
    protected Document createDefaultModel() {

        // Method to create default model
        return new NumericPlainDocument(1);
    }

}